/*********************************************************************
 *
 * Class   : DemoFrame
 *
 * Author  : Georg Walkner
 *
 * Purpose : Responsible for the 
 *           Layout of the Application
 *
 * Date    : 05-Mar-2002
 *
 ********************************************************************/
package EIB;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;

import com.eiblet.bus.*;
import com.eiblet.bus.eib.*;
import com.eiblet.util.*;


public class DemoFrame extends JFrame
    implements ActionListener, MenuListener
{
    private EIBLampButton  LampButton1;
    private EIBLampButton  LampButton2;
    private EIBLampButton  LampButton3;
    private EIBLampButton  LampButton4;
    private DimmerPanel    fDimmerPanel1;
    private DimmerPanel    fDimmerPanel2;

    private JButton    CloseButton;
    private JTextArea  LogArea;
    private EIBLowLevelGroupObjectBusCommunicator fEIBLowLevelBC;


    public DemoFrame(EIBLowLevelGroupObjectBusCommunicator pEIBLowLevelBC)
    {
	super("Demo Program");
	setSize(400, 300);

	fEIBLowLevelBC = pEIBLowLevelBC;

	initComponents();           // Erstellt gesamtes Layout
	this.setLocation(540, 0);
	this.show();
    }


    public void handleEvent
	(EIBLowLevelGroupObjectBusCommunicatorEvent pEvent)
    {
	LampButton1.handleEvent(pEvent);
	LampButton2.handleEvent(pEvent);
	LampButton3.handleEvent(pEvent);
	LampButton4.handleEvent(pEvent);
	fDimmerPanel1.handleEvent(pEvent);
	fDimmerPanel2.handleEvent(pEvent);
    }



    public void actionPerformed(ActionEvent evt)
    {
	String arg    = evt.getActionCommand();

	if(arg.equals("Close")) {
	    this.dispose();
	}	
    }


    private void initComponents()
    {
	LampButton1        = new EIBLampButton("Lamp 1",
					   new EIBGroupAddress(1,1,1),
					   new EISType(EISType.EIS_TYPE_SWITCH),
					   fEIBLowLevelBC);
	LampButton2        = new EIBLampButton("Lamp 2",
					   new EIBGroupAddress(1,1,2),
					   new EISType(EISType.EIS_TYPE_SWITCH),
					   fEIBLowLevelBC);
	LampButton3        = new EIBLampButton("Lamp 3",
					   new EIBGroupAddress(1,1,3),
					   new EISType(EISType.EIS_TYPE_SWITCH),
					   fEIBLowLevelBC);
	LampButton4        = new EIBLampButton("Lamp 4",
					   new EIBGroupAddress(1,1,4),
					   new EISType(EISType.EIS_TYPE_SWITCH),
					   fEIBLowLevelBC);
	EIBGroupAddress[] foo = {
	    new EIBGroupAddress(1, 1, 5),
	    new EIBGroupAddress(1, 3, 1),
	    new EIBGroupAddress(1, 2, 1)};
	fDimmerPanel1 = new DimmerPanel("Dimmer 1", foo, fEIBLowLevelBC);

	
	foo[0] = new EIBGroupAddress(1, 1, 6);
	foo[1] = new EIBGroupAddress(1, 3, 2);
	foo[2] = new EIBGroupAddress(1, 2, 2);
	fDimmerPanel2 = new DimmerPanel("Dimmer 2", foo, fEIBLowLevelBC);

        CloseButton        = new JButton("Close");

	JPanel panel       = new JPanel(new GridBagLayout());
       	Container cont     = getContentPane();
	cont.add(panel);

        panel.add(LampButton1, new GridBagConstraints(0, 0, 
						      1, 1, 
						      10.0, 10.0,
						      GridBagConstraints.CENTER,
						      GridBagConstraints.HORIZONTAL,
						      new Insets(10, 10, 10, 10), 
						      0, 0));

        panel.add(LampButton2, new GridBagConstraints(0, 1, 
						      1, 1, 
						      10.0, 10.0,
						      GridBagConstraints.CENTER,
						      GridBagConstraints.HORIZONTAL,
						      new Insets(10, 10, 10, 10), 
						      0, 0));

        panel.add(LampButton3, new GridBagConstraints(1, 0, 
						      1, 
						      1, 
						      10.0, 10.0,
						      GridBagConstraints.CENTER,
						      GridBagConstraints.HORIZONTAL,
						      new Insets(10, 10, 10, 10), 
						      0, 0));

        panel.add(LampButton4, new GridBagConstraints(1, 1, 1,
						      //GridBagConstraints.REMAINDER,  
						      1, 
						      10.0, 10.0,
						      GridBagConstraints.CENTER,
						      GridBagConstraints.HORIZONTAL,
						      new Insets(10, 10, 10, 10), 
						      0, 0));

        panel.add(fDimmerPanel1, new GridBagConstraints(0, 2, 
							1, 1, 
							10.0, 10.0,
							GridBagConstraints.CENTER,
							GridBagConstraints.BOTH,
							new Insets(10, 10, 10, 10), 
							0, 0));

        panel.add(fDimmerPanel2, new GridBagConstraints(1, 2, 
							GridBagConstraints.REMAINDER, 
							1, 
							10.0, 10.0,
							GridBagConstraints.CENTER,
							GridBagConstraints.BOTH,
							new Insets(10, 10, 10, 10), 
							0, 0));


	CloseButton.addActionListener(this);
        panel.add(CloseButton, new GridBagConstraints(1, 6, 
						     GridBagConstraints.REMAINDER, 
						     GridBagConstraints.REMAINDER, 
						     10.0, 10.0,
						     GridBagConstraints.SOUTHEAST,
						     GridBagConstraints.NONE,
						     new Insets(10, 10, 10, 10), 
						     0, 0));


	// Menue erzeugen das is nur proforma
	JMenuBar mbar = new JMenuBar();
	JMenu fileMenu = new JMenu("File");
	JMenuItem closeItem = new JMenuItem("Close");

	this.setJMenuBar(mbar);
	fileMenu.addMenuListener(this);
	closeItem.addActionListener(this);
	fileMenu.add(closeItem);
	mbar.add(fileMenu);
    }

    public void menuSelected(MenuEvent evt) { }
    public void menuDeselected(MenuEvent evt) { }
    public void menuCanceled(MenuEvent evt)   { }

}




