/*********************************************************************
 *
 * Class   : DimmerPanel.java
 *
 * Author  : Georg Walkner
 *
 * Purpose : Panel Extension
 *
 * Date    : 05-Mar-2002
 *
 ********************************************************************/
package EIB;

import java.io.*;

import javax.swing.*;
//import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;

import com.eiblet.bus.*;
import com.eiblet.bus.eib.*;
import com.eiblet.util.*;


public class DimmerPanel extends JPanel
    implements BusCommunicatorStatusEventListener
{
    private EIBLowLevelGroupObjectBusCommunicator fEIBLowLevelBC = null;
    private EIBLampButton[] LampButton;
    private JButton         DownButton;
    private JButton         UpButton;
 
    private EIBGroupObject fGroupObject;

    private static final byte DOWNEN = 1; // Dimmer Down Enabled
    private static final byte DOWNDI = 0; // Dimmer Down Disabled
    private static final byte UPEN   = 9; // Dimmer Up Enabled
    private static final byte UPDI   = 8; // Dimmer Up Disabled
    
    public DimmerPanel(String pPanelName, 
		       EIBGroupAddress[] pEIBGroupAddress, 
		       EIBLowLevelGroupObjectBusCommunicator pEIBLowLevelBC)
    {
	super(new GridBagLayout());
	fEIBLowLevelBC = pEIBLowLevelBC;
	fEIBLowLevelBC.addBusCommunicatorStatusEventListener(this);
	fGroupObject = new EIBGroupObject(pEIBGroupAddress[2], 
					  new EISType(EISType.EIS_TYPE_SCALING_LUX),
					  pEIBLowLevelBC);
	EIBLampButton[] bar = {
	    new EIBLampButton(pPanelName + " On/Off", 
			      pEIBGroupAddress[0], 
			      new EISType(EISType.EIS_TYPE_SWITCH),
			      pEIBLowLevelBC),
	    new EIBLampButton(pPanelName + " Enable", 
			      pEIBGroupAddress[1], 
			      new EISType(EISType.EIS_TYPE_SWITCH),
			      pEIBLowLevelBC) };

	LampButton = bar;

	initComponents();
    }
    
    
    public void handleEvent(EIBLowLevelGroupObjectBusCommunicatorEvent pEvent)
    {
	if(fGroupObject.getEIBGroupAddress().equals(pEvent.getEIBGroupAddress())) {

	    fGroupObject.setData(pEvent.getData());
	    switch(fGroupObject.getData()[0]) {
	    case DOWNEN:
		DownButton.setBackground(Color.yellow); break;

	    case DOWNDI:
		DownButton.setBackground(Color.white);  break;

	    case UPEN:
		UpButton.setBackground(Color.yellow); break;

	    case UPDI:
		UpButton.setBackground(Color.white);  break;

	    default:
		System.out.println("Error");
		break;
	    }
	}

	for(int i=0; i<2; ++i) {
	    LampButton[i].handleEvent(pEvent);
	}
    }
    
    public void handleBusCommunicatorStatusEvent(BusCommunicatorStatusEvent pEvent){
	if(BusCommunicatorStatusEvent.STATUS_DISCONNECTED == pEvent.getStatus()){

	}
    }   

    private void initComponents()
    {
	UpButton   = new JButton("->");
	UpButton.setBackground(Color.white);
	DownButton = new JButton("<-");
	DownButton.setBackground(Color.white);

	UpButton.addMouseListener(new MouseAdapter() {
		public void mousePressed(MouseEvent e) 	{ 
		    if((e.getModifiers() & MouseEvent.BUTTON1_MASK) == MouseEvent.BUTTON1_MASK) {
			try{byte[] foo = {UPEN};
			    fGroupObject.writeGroupData(foo);
			} catch(IOException ex){
			    System.out.println(ex.getMessage());
			}
		    }
		}
		public void mouseReleased(MouseEvent e) {
		    if((e.getModifiers() & MouseEvent.BUTTON1_MASK) == MouseEvent.BUTTON1_MASK) {
			try{byte[] foo = {UPDI};
			    fGroupObject.writeGroupData(foo);
			} catch(IOException ex){
			    System.out.println(ex.getMessage());
			}
		    }	    
		}} );
	
	DownButton.addMouseListener(new MouseAdapter()  {
		public void mousePressed(MouseEvent e)  { 
		    if((e.getModifiers() & MouseEvent.BUTTON1_MASK) == MouseEvent.BUTTON1_MASK) {
			try{byte[] foo = {DOWNEN};
			    fGroupObject.writeGroupData(foo);
			} catch(IOException ex){
			    System.out.println(ex.getMessage());
			}
		    }
		}
		public void mouseReleased(MouseEvent e)	{
		    if((e.getModifiers() & MouseEvent.BUTTON1_MASK) == MouseEvent.BUTTON1_MASK) {
			try{byte[] foo = {DOWNDI};
			    fGroupObject.writeGroupData(foo);
			} catch(IOException ex){
			    System.out.println(ex.getMessage());
			}
		    }	    
		}} );
	


	//UpButton.addChangeListener(this);
	//DownButton.addChangeListener(this);

	for(int i=0; i<2; ++i) {
	    this.add(LampButton[i], new GridBagConstraints(0, i, 
							   GridBagConstraints.REMAINDER,  
							   1, 
							   100.0, 100.0,
							   GridBagConstraints.CENTER,
							   GridBagConstraints.HORIZONTAL,
							   new Insets(0, 0, 0, 0), 
							   0, 0));
	}

	this.add(DownButton, new GridBagConstraints(0, 2, 
						    1,
						    GridBagConstraints.REMAINDER,  
						    100.0, 100.0,
						    GridBagConstraints.CENTER,
						    GridBagConstraints.HORIZONTAL,
						    new Insets(0, 0, 0, 0), 
						    0, 0));

	this.add(UpButton, new GridBagConstraints(1, 2, 
						  GridBagConstraints.REMAINDER,  
						  GridBagConstraints.REMAINDER,  
						  100.0, 100.0,
						  GridBagConstraints.CENTER,
						  GridBagConstraints.HORIZONTAL,
						  new Insets(0, 0, 0, 0), 
						  0, 0));

    }

}
