/*********************************************************************
 *
 * Class   : EIBFrame
 *
 * Author  : Georg Walkner
 *
 * Purpose : Responsible for the 
 *           Layout of the Application
 *
 * Date    : 05-Mar-2002
 *
 ********************************************************************/
package EIB;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;

import com.eiblet.bus.*;
import com.eiblet.bus.eib.*;
import com.eiblet.util.*;


public class EIBFrame extends JFrame
    implements ActionListener, MenuListener, 
    EIBLowLevelGroupObjectBusCommunicatorEventListener,  
    BusCommunicatorStatusEventListener 
{
    private DemoFrame  demoframe = null;
    private String     LogString = "";

    private JButton    SendButton;   
    private JButton    QuitButton;
    private JButton    ConnectButton;
    private JButton    DisConnButton;
    private JButton    DemoButton;
   
    private JTextField GroupAdrField;
    private JTextField DataField;
    private JTextField ConnectField;

    private EIBTableModel LogData;
    private EIBLowLevelGroupObjectBusCommunicator fEIBLowLevelBC;


    public EIBFrame(EIBLowLevelGroupObjectBusCommunicator pEIBLowLevelBC)
    {
	super("EIB Program");
	setSize(540, 300);

	fEIBLowLevelBC = pEIBLowLevelBC;
	fEIBLowLevelBC.setEIBLowLevelGroupObjectBusCommunicatorEventListener(this);
	fEIBLowLevelBC.addBusCommunicatorStatusEventListener(this);

	addWindowListener(new WindowAdapter() 
	    { 
		public void windowClosing(WindowEvent e) 
		{ 
		    fEIBLowLevelBC.close();
		    System.exit(0); 
		}
	    } );

	LogData = new EIBTableModel();
	
	initComponents();           // Erstellt gesamtes Layout
	setLocation(0, 0);

	demoframe = new DemoFrame(fEIBLowLevelBC);
    }


    public void handleEIBLowLevelGroupObjectBusCommunicatorEvent
	(EIBLowLevelGroupObjectBusCommunicatorEvent pEvent)
    {
	EIBGroupObject foo = new EIBGroupObject(pEvent.getEIBGroupAddress(),
						new EISType(EISType.EIS_TYPE_SWITCH),
						fEIBLowLevelBC);
	foo.setData(pEvent.getData());
	 
	LogData.addLogEntry(foo);

	if(demoframe != null) {
	    demoframe.handleEvent(pEvent);
	}
    }

    public void handleBusCommunicatorStatusEvent(BusCommunicatorStatusEvent pEvent)
    {
	System.out.println("Bus Event " + pEvent);
	System.out.println("connected:" + fEIBLowLevelBC.isConnected());
	
	if(fEIBLowLevelBC.isConnected()){
	    ConnectField.setText("connected");
	}
	else{
	    ConnectField.setText("disconnected");
	}
    }


    public void actionPerformed(ActionEvent evt)
    {
	String arg    = evt.getActionCommand();
	Object source = evt.getSource();

	if(source == SendButton) {
	    EIBGroupAddress fooadr = new EIBGroupAddress
		(GroupAdrField.getText(),
		 EIBGroupAddress.FORMAT_THREE_PART);
	    String foostr = DataField.getText();
	    byte[] bar = new byte[foostr.length()];
	    for(int i=0; i<foostr.length(); ++i) {
		bar[i] = (byte)(foostr.charAt(i) - 48); // Ugly hack
	    }
	    
	    EIBGroupObject eibobj = new EIBGroupObject(fooadr,
						       new EISType(EISType.EIS_TYPE_SWITCH),
						       fEIBLowLevelBC);
	    try {
		eibobj.writeGroupData(bar);
	    }
	    catch(Exception e) {
		System.out.println("Problem!");
	    }
	}
	else if(source == ConnectButton) {
	    fEIBLowLevelBC.open();
	}
	else if(source == DisConnButton) {
	    fEIBLowLevelBC.close();
	}
	else if(arg.equals("Demo")) {
	    //if(demoframe == null) {
		demoframe = new DemoFrame(fEIBLowLevelBC);
		//}
	}
	else if(arg.equals("Quit")) {
	    fEIBLowLevelBC.close();
	    System.exit(0);
	}	
    }


    private void initComponents()
    {
        SendButton         = new JButton("Send");
	SendButton.addActionListener(this);

	ConnectButton      = new JButton("Connect");
	ConnectButton.addActionListener(this);

	DisConnButton      = new JButton("Disconnect");
	DisConnButton.addActionListener(this);

	DemoButton         = new JButton("Demo");
	DemoButton.addActionListener(this);

        QuitButton         = new JButton("Quit");
	QuitButton.addActionListener(this);

        GroupAdrField      = new JTextField("1/1/1");
	DataField          = new JTextField();
	ConnectField       = new JTextField("connected?");

	JLabel GroupAdrLabel = new JLabel("Group Adress:");
	JLabel DataLabel     = new JLabel("Data:");
	JLabel LogLabel      = new JLabel("Log:");
	JLabel ConnectLabel  = new JLabel("Connection:");

	JTable LogTable    = new JTable(LogData);
	JScrollPane LogPane= new JScrollPane(LogTable);

	JPanel centerp     = new JPanel(new GridBagLayout());

       	Container cont     = getContentPane();
	cont.add(centerp);


        centerp.add(GroupAdrLabel, new GridBagConstraints(0, 0, 1, 1, 
							  0.0, 0.0,
							  GridBagConstraints.EAST,
							  GridBagConstraints.NONE,
							  new Insets(10, 10, 10, 10), 
							  0, 0));

        centerp.add(GroupAdrField, new GridBagConstraints(1, 0, 
							  1, 1, 
							  10.0, 10.0,
							  GridBagConstraints.CENTER,
							  GridBagConstraints.HORIZONTAL,
							  new Insets(10, 10, 10, 10), 
							  10, 0));

        centerp.add(DataLabel, new GridBagConstraints(0, 1, 1, 1, 
						      0.0, 0.0,
						      GridBagConstraints.EAST,
						      GridBagConstraints.NONE,
						      new Insets(10, 10, 10, 10), 
						      0, 0));

        centerp.add(DataField, new GridBagConstraints(1, 1, 
						      1, 1, 
						      10.0, 10.0,
						      GridBagConstraints.CENTER,
						      GridBagConstraints.HORIZONTAL,
						      new Insets(10, 10, 10, 10), 
						      0, 0));

        centerp.add(ConnectLabel, new GridBagConstraints(0, 2, 1, 1, 
						      0.0, 0.0,
						      GridBagConstraints.EAST,
						      GridBagConstraints.NONE,
						      new Insets(10, 10, 10, 10), 
						      0, 0));

        centerp.add(ConnectField, new GridBagConstraints(1, 2, 1, 1, 
						      0.0, 0.0,
						      GridBagConstraints.CENTER,
						      GridBagConstraints.HORIZONTAL,
						      new Insets(10, 10, 10, 10), 
						      0, 0));

        centerp.add(DataField, new GridBagConstraints(1, 1, 
						      1, 1, 
						      10.0, 10.0,
						      GridBagConstraints.CENTER,
						      GridBagConstraints.HORIZONTAL,
						      new Insets(10, 10, 10, 10), 
						      0, 0));


        centerp.add(SendButton, new GridBagConstraints(2, 1, 1,
						       //GridBagConstraints.REMAINDER, 
						       1, 
						      0.0, 0.0,
						      GridBagConstraints.CENTER,
						      GridBagConstraints.HORIZONTAL,
						      new Insets(5, 10, 5, 10), 
						      0, 0));

        centerp.add(DemoButton, new GridBagConstraints(3, 1, 
						       GridBagConstraints.REMAINDER, 
						       1, 
						       0.0, 0.0,
						       GridBagConstraints.CENTER,
						       GridBagConstraints.HORIZONTAL,
						       new Insets(5, 10, 5, 10), 
						       0, 0));

        centerp.add(ConnectButton, new GridBagConstraints(2, 2, 1, 1, 
							    //GridBagConstraints.REMAINDER, 
							  0.0, 0.0,
							  GridBagConstraints.CENTER,
							  GridBagConstraints.HORIZONTAL,
							  new Insets(5, 10, 5, 10), 
							  0, 0));

        centerp.add(DisConnButton, new GridBagConstraints(3, 2, 
							  GridBagConstraints.REMAINDER, 
							  1,  
							  0.0, 0.0,
							  GridBagConstraints.CENTER,
							  GridBagConstraints.HORIZONTAL,
							  new Insets(5, 10, 5, 10), 
							  0, 0));

        centerp.add(QuitButton, new GridBagConstraints(3, 3, 
						      GridBagConstraints.REMAINDER, 
						      1, 
						      0.0, 0.0,
						      GridBagConstraints.CENTER,
						      GridBagConstraints.HORIZONTAL,
						      new Insets(5, 10, 5, 10), 
						      0, 0));

        centerp.add(LogLabel, new GridBagConstraints(0, 4, 
						     GridBagConstraints.REMAINDER, 
						     1, 
						     0.0, 0.0,
						     GridBagConstraints.WEST,
						     GridBagConstraints.NONE,
						     new Insets(10, 0, 5, 0), 
						     0, 0));
	
        centerp.add(LogPane, new GridBagConstraints(0, 5, 
						   GridBagConstraints.REMAINDER, 
						   GridBagConstraints.REMAINDER, 
						   10.0, 100.0,
						   GridBagConstraints.CENTER,
						   GridBagConstraints.BOTH,
						   new Insets(0, 0, 0, 0), 
						   0, 100));


	// Menue erzeugen das is nur proforma
	JMenuBar mbar = new JMenuBar();
	JMenu fileMenu = new JMenu("File");
	JMenuItem demoItem = new JMenuItem("Demo");
	JMenuItem quitItem = new JMenuItem("Quit");

	this.setJMenuBar(mbar);
	fileMenu.addMenuListener(this);
	demoItem.addActionListener(this);
	fileMenu.add(demoItem);
	quitItem.addActionListener(this);
	fileMenu.add(quitItem);
	mbar.add(fileMenu);
    }

    public void menuSelected(MenuEvent evt) { }
    public void menuDeselected(MenuEvent evt) { }
    public void menuCanceled(MenuEvent evt)   { }
}




