/*
     LinuxBCU1EIBBusCommunicator - Eiblet BusCommunicator for low level EIB driver BCU 1

     Copyright (C) 2000 Matthias Wabel <e9425054@student.tuwien.ac.at>,
                        Bernd Thallner <bernd@kangaroo.at>

     License have to be egregioused. Presently only limited version for test purpose,
     no public "free" use!
     see http://www.eiblet.com/bus/apidoc/ for documentation.
*/

//package com.eiblet.linuxbcu1eibbuscommunicator;
package EIB;

import java.io.*;
import java.util.*;

import com.eiblet.bus.*;
import com.eiblet.bus.eib.*;
import com.eiblet.util.*;

public final class LinuxBCU1EIBBusCommunicator
    implements EIBLowLevelGroupObjectBusCommunicator {
    
    public static final String LinuxBCU1EIBBusCommunicatorName = 
	"Eiblet BusCommunicator for Low Level EIB driver BCU 1";
    public static final String LinuxBCU1EIBBusCommunicatorVersion = "0.1";
    
    // Low Level EIB driver device
    public static final String eibDevice = "/dev/eib";
    //public static final String eibDevice = "home/georg/test";
    
    
    // maximum time to wait in 20 milliseconds for confirm and indication
    public static final int TIMEOUT = 25;
    
    public static final int WRITE_GROUP_DATA = 0,
	READ_GROUP_DATA = 1, 
	RESPONSE_GROUP_DATA = 2, 
	OPEN = 3, 
	CLOSE = 4;
    
    private EIBLowLevelGroupObjectBusCommunicator pSource = this;
    private EIBLowLevelGroupObjectBusCommunicatorEventListener eventListener = null;
    private Vector vStatusEventListeners = new Vector();
    private FileWriter eibWriter;
    private FileReader eibReader;
    private byte con[] = new byte[32];
    private byte valueResponse[] = new byte[32];
    private boolean opened = false;
    private Thread eventListenerThread;
    
    class EventListenerThread
	implements Runnable {
	int len;
	char[] cbuf = new char[32];
	
	public void run() {

	    while(opened) {
		try {
		    len = eibReader.read(cbuf);
		    if(len > 32) {
			sendMessage(BusCommunicatorStatusEvent.STATUS_MESSAGE, "unallowed data block length (" + len + ")");
		    } 
		    else if(len > 0) {
			if(cbuf[0] == 0x49 || cbuf[0] == 0x4e || cbuf[0] == 0x4b) { 
			    // L_Data.ind, L_Data.con, PC_GET_VAL.con, PC_GET_VAL.con
			    byte[] pData;
			    int apci;
			    
			    if(cbuf[0] == 0x4e || cbuf[0] == 0x4b) // L_Data.con, PC_GET_VAL.con
				for(int i = len-1; i>=0; i--)
				    con[i] = (byte)cbuf[i];
			    if(cbuf[0] != 0x4b) { // L_Data.ind, L_Data.con
				if((cbuf[6]&0x0f)==1)
				    pData = new byte[]{(byte)(cbuf[8]&0x3f)};
				else {
				    pData = new byte[(cbuf[6]&0x0f)-1];
				    for(int i = 0; i < (cbuf[6]&0x0f)-1; i++)
					pData[i] = (byte)cbuf[9+i];
				}

				apci = ((((int)cbuf[7]&0x03)<<8)|(cbuf[8]&0xc0));
				if(cbuf[0] == 0x49 && apci == 
				   EIBLowLevelGroupObjectBusCommunicatorEvent.TYPE_VALUE_RESPONSE) { 
				    //  L_Data.ind
				    for(int i = len-1; i>=0; i--)
					valueResponse[i] = (byte)cbuf[i];
				}
				if(eventListener != null)
				    eventListener.handleEIBLowLevelGroupObjectBusCommunicatorEvent(
					 new EIBLowLevelGroupObjectBusCommunicatorEvent(pSource,
											new EIBGroupAddress((cbuf[4]<<8)|cbuf[5]),
											(cbuf[1]&0x0c)>>2,
											(cbuf[6]&0x70)>>4,
											apci,
											pData));
			    }
			} else {
			    String s = "";
			    for(int i = 0; i<len; i++)
				s += Integer.toHexString(cbuf[i]) + " ";
			    sendMessage(BusCommunicatorStatusEvent.STATUS_MESSAGE, "unexpected data block received (" + s + ")");
			}
		    }
		} catch (Exception e) {
		    e.printStackTrace();
		    System.out.println("cdd");
		}
	    }
	}
    }
    
    public LinuxBCU1EIBBusCommunicator() {
	System.out.println(LinuxBCU1EIBBusCommunicatorName + ", version=" + LinuxBCU1EIBBusCommunicatorVersion);
    }

    public void writeGroupData(EIBGroupAddress pEIBGroupAddress, int pPriority, int pRoutingCount, boolean pLess7Bits, byte[] pData)
	throws IllegalArgumentException, IOException {
	operationGroupData(WRITE_GROUP_DATA, pEIBGroupAddress, pPriority, pRoutingCount, pLess7Bits, pData);
    }

    public byte[] readGroupData(EIBGroupAddress pEIBGroupAddress, int pPriority, int pRoutingCount)
	throws IllegalArgumentException, IOException {
	return operationGroupData(READ_GROUP_DATA, pEIBGroupAddress, pPriority, pRoutingCount, true, null);
    }

    public void responseGroupData(EIBGroupAddress pEIBGroupAddress, int pPriority, int pRoutingCount, boolean pLess7Bits, byte[] pData)
	throws IllegalArgumentException, IOException {
	operationGroupData(RESPONSE_GROUP_DATA, pEIBGroupAddress, pPriority, pRoutingCount, pLess7Bits, pData);
    }

    public void open() {
	try {
	    operationGroupData(OPEN, null, 0, 0, true, null);
	} catch (Exception e) { }
    }
    
    public void close() {
	try {
	    operationGroupData(CLOSE, null, 0, 0, true, null);
	} catch (Exception e) { }
    }
    
    public boolean isConnected() {
	return opened;
    }
    
    private synchronized byte[] operationGroupData(int operation, 
						   EIBGroupAddress pEIBGroupAddress, 
						   int pPriority, 
						   int pRoutingCount, 
						   boolean pLess7Bits, 
						   byte[] pData)
	throws IllegalArgumentException, IOException 
    {
	switch(operation) {

	case OPEN:
	    
	    if(opened)
		sendMessage(BusCommunicatorStatusEvent.STATUS_MESSAGE, "Buscommunicator already opened");
	    else {
		try {
		    eibWriter = new FileWriter(eibDevice);
		    eibReader = new FileReader(eibDevice);
		    
		    char[] data = new char[]{0x46, 0x01, 0x00, 0x60, 0x12}; // System State, Link-Layer
		    eibWriter.write(data, 0, 5);
		    eibWriter.flush();
		    data = new char[]{0x46, 0x01, 0x01, 0x16, 0x00}; // AdrTab, length=0, all group-addresses accepted
		    eibWriter.write(data, 0, 5);
		    eibWriter.flush();
		    opened=true;
		    eventListenerThread = new Thread(new EventListenerThread());
		    eventListenerThread.start();
		    sendMessage(BusCommunicatorStatusEvent.STATUS_CONNECTED, "LinuxBCU1EIBBusCommunicator successful opened");
		} catch(java.io.IOException e) {
		    opened=false;
		    sendMessage(BusCommunicatorStatusEvent.STATUS_MESSAGE, "open failed");
		}
	    }
	    return null;
	    
	case CLOSE:
	    
	    if(!opened)
		sendMessage(BusCommunicatorStatusEvent.STATUS_MESSAGE, "Buscommunicator not opened");
	    else {
		try {
		    opened = false;
		    
		    con[0] = 0;
		    char[] data = new char[]{0x4b, 0x01, 0x00, 0x60}; // PC_GET_VAL.req, System State
		    eibWriter.write(data, 0, 4);
		    eibWriter.flush();
		    
		    for(int i = 0; (con[0] == 0) && (i < TIMEOUT); i++) {
			try {
			    Thread.currentThread().sleep(20);
			} catch (Exception e) { }
		    }
		    if(con[0] != 0x4b) {
			opened=true;
			sendMessage(BusCommunicatorStatusEvent.STATUS_MESSAGE, "close failed");
		    } else {
			eventListenerThread = null;
			data = new char[]{0x46, 0x01, 0x00, 0x60, 0xc0}; // PC_SET_VAL.req, System State, Reset
			eibWriter.write(data, 0, 5);
			eibWriter.flush();
			eibWriter.close();
			eibWriter = null;
			eibReader.close();
			eibReader = null;
			sendMessage(BusCommunicatorStatusEvent.STATUS_DISCONNECTED, "LinuxBCU1EIBBusCommunicator closed");
		    }
		} catch(java.io.IOException e) {
		    sendMessage(BusCommunicatorStatusEvent.STATUS_MESSAGE, "close failed");
		}
	    }
	    return null;
	    
	case WRITE_GROUP_DATA:
	case RESPONSE_GROUP_DATA: {
	    int len, apci;
	    char out[] = new char[32];
	    
	    if(!opened)
		throw new IOException("Buscommunicator not opened");
	    if(operation == WRITE_GROUP_DATA)
		apci = 0x80;
	    else
		apci = 0x40;
	    if(pLess7Bits) {
		len = 1;
		out[8] = (char)(apci|(pData[0]&0x3f));
	    } else { // !pLess7Bits
		len = (1+pData.length)&0x0f;
		out[8] = (char)apci;
		for(int i = 0; (i < pData.length) && (9+i < 32); i++) {
		    out[9+i] = (char)pData[i];
		}
	    }
	    
	    out[0] = 0x11; // L_Data.req
	    out[1] = (char)(pPriority<<2);
	    out[2] = 0x00;
	    out[3] = 0x00;
	    out[4] = (char)((pEIBGroupAddress.getGroupAddress()&0xff00)>>8);
	    out[5] = (char)(pEIBGroupAddress.getGroupAddress()&0x00ff);
	    out[6] = (char)(0x80|((pRoutingCount&0x07)<<4)|len);
	    out[7] = 0x00;
	    
	    con[0] = 0;
	    eibWriter.write(out, 0, 8 + len);
	    eibWriter.flush();
	    
	    for(int i = 0; (con[0] == 0) && (i < TIMEOUT); i++) {
		try {
		    Thread.currentThread().sleep(20);
		} catch (Exception e) { }
	    }
	    
	    if(con[0] != 0x4e) {
		throw new IOException("timeout L_Data.con message in writeGroupData");
	    } else {
		for(int i = 4; i < 7+len; i++)
		    if(con[i]!=(byte)out[i])
			throw new IOException("error in L_Data.con message");
		return null;
	    }
	}
	    
	case READ_GROUP_DATA: {
	    char out[] = new char[32];
	    int len;
	    byte[] data;
	    
	    if(!opened)
		throw new IOException("Buscommunicator not opened");
	    out[0] = 0x11; // L_Data.req
	    out[1] = (char)(pPriority<<2);
	    out[2] = 0x00;
	    out[3] = 0x00;
	    out[4] = (char)((pEIBGroupAddress.getGroupAddress()&0xff00)>>8);
	    out[5] = (char)(pEIBGroupAddress.getGroupAddress()&0x00ff);
	    out[6] = (char)(0x80|((pRoutingCount&0x07)<<4)|0x01);
	    out[7] = 0x00;
	    out[8] = 0x00;
	    
	    con[0] = 0;
	    valueResponse[0] = 0;
	    eibWriter.write(out, 0, 9);
	    eibWriter.flush();
	    
	    for(int i = 0; ((con[0] == 0) || (valueResponse[0] == 0)) && (i < TIMEOUT); i++) {
		try {
		    Thread.currentThread().sleep(20);
		} catch (Exception e) { }
	    }
	    
	    if(con[0] != 0x4e) {
		throw new IOException("timeout L_Data.con message in readGroupData");
	    } else if(valueResponse[0] == 0) {
		throw new IOException("timeout Value Response message in readGroupData");
	    } else {
		for(int i = 4; i < 8; i++)
		    if(con[i]!=(byte)out[i])
			throw new IOException("error in L_Data.con message");
		if((valueResponse[4]!=out[4]) || (valueResponse[5]!=out[5]))
		    throw new IOException("unexpected Value Response message");
		
		len = (valueResponse[6]&0x0f);
		if(len == 1) {
		    data = new byte[]{(byte)(valueResponse[8]&0x3f)};
		} else {
		    data = new byte[len-1];
		    for(int i = 0; i < len-1; i++)
			data[i] = valueResponse[9 + i];
		}
		return data;
	    }
	}
	    
	default:
	    throw new IOException("function not yet implemented");
	}
    }
    
    private void sendMessage(int status, String message) {
	BusCommunicatorStatusEvent e = new BusCommunicatorStatusEvent(this, status, message);
	
	for(int i = 0; i < vStatusEventListeners.size(); i++) {
	    try {
		((BusCommunicatorStatusEventListener)vStatusEventListeners.elementAt(i)).handleBusCommunicatorStatusEvent(e);
	    } catch (Exception ex) {
		ex.printStackTrace();
	    }
	}
    }
    
    public void setEIBLowLevelGroupObjectBusCommunicatorEventListener
	(EIBLowLevelGroupObjectBusCommunicatorEventListener pListener) 
    {
	eventListener = pListener;
    }
    
    public void addBusCommunicatorStatusEventListener(BusCommunicatorStatusEventListener pListener) {
	if(!vStatusEventListeners.contains(pListener))
	    vStatusEventListeners.add(pListener);
    }
    
    public void removeBusCommunicatorStatusEventListener(BusCommunicatorStatusEventListener pListener) {
	if(vStatusEventListeners.contains(pListener))
	    vStatusEventListeners.remove(pListener);
    }
    
    public void setParameters(ParameterCollection pParameterCollection) { }
}
